<!-- -*- mode: markdown; mode: flyspell -*- -->

# bash-ini library

A library to query [ini style](https://en.wikipedia.org/wiki/INI_file)
configuration files. Currently there is no support to write or change
files on disk.

## Security

This library currently makes heavy use of `eval`. This can not easily
change until bash 4.2 support is dropped. Until that it is probably a
good idea not using it in privileged scripts to parse ini files that
can be modified by untrusted users.

## Naming

To use the bash-ini, first the contents of an ini file must be loaded
into memory. There it is called database (db). This database is read
only and will not follow any changes on disk.

## ini file parsing

The ini file format is not standardized. bash-ini parses the following
file format:

1. Empty lines or comment lines (`^([[:space:]]*|[\;\#].*)$`) are ignored.
2. Any other line must be either a key value pair or a section definition
3. Section definitions must have the format `^\[([[:alnum:]_\-]([[:alnum:]\ _\-]*[[:alnum:]_\-])?)\][[:space:]]*$`
4. Key value pairs must have the format `^[[:space:]]*([[:alnum:]]([[:alnum:]\ _\-]*[[:alnum:]_\-])?)[[:blank:]]*=[[:blank:]]*(.*)[[:blank:]]*$`

## API Docu

```bash
# bi_db_load <db-name> <ini-file>
# loads contents of ini-file
# db-name must match ^[[:alnum:]_]+$ and must be currently unset
# errors: 1=file not readable, 2=variable not unset, 3=parse error
```

```bash
# bi_db_destroy <db-name>
# destroys loaded ini db
```

```bash
# bi_db_is_loaded <db-name>
# check if db is loaded
```

```bash
# bi_key_exists <db-name> <key> [<section>]
# check if a key exists
# if no section is specified only global context is searched
```

```bash
# bi_key_exists_with_default <db-name> <key> <section>
# like bi_key_exists but if section does not exist or contain key it will
# search in global context
```

```bash
# bi_section_exists <db-name> <section>
# check if a section exists
```

```bash
# bi_get_section_list <retval-name> <db-name>
# return a list of all sections as array into global array <retval-name>
# retval-name must match ^[[:alpha:]][[:alnum:]_]*$
# note: the order is undefined
```

```bash
# bi_get_value <retval-name> <db-name> <key> [ <section> ]
# return value for <key> in <db-name> in optional <section>
# note: if <section> is specified key must exist in it
# note: <retval-name> must not start with '_'
```

```bash
# bi_get_value_with_default <variablename> <db-name> <_key> <_section>
# return value for <key> in <db-name> in <section> or use default
# note: <retval-name> must not start with '_'
```

## Examples

```bash
> brp_expand_ranges "" "" "" 1 2 3
1 2 3

> brp_expand_ranges "pre" "post" "" 1 2 3 
pre1post pre2post pre3post

> brp_expand_ranges "" "post" "" 1-3
1post 2post 3post

> brp_expand_ranges "" "post" "" 1-3 somestring 3
1post 2post 3post somestring 3post

BRB_RANGE_MACROS=([cow]=1-3 [cat]=2-5

> brp_expand_ranges "" "" "" cow
1 2 3

> brp_expand_ranges "" "" "" cat cow
2 3 4 5 1 2 3

BRB_RANGE_MACROS=([cow1]="1-3" [cow2]="4-5" [herd]="cow1 cow2")

> brp_expand_ranges "mooh" "" "" herd
mooh1 mooh2 mooh3 mooh4 mooh5
```
